namespace eval mailing_list::util {

    ad_proc -private reset_cached_package_urls {} {
	Caches the urls of the site-wide package instances of
	categories and users-selection
    } {
	catch {nsv_unset ml_cached_urls}

	db_foreach get_site_wide_package_urls {} {
	    nsv_set ml_cached_urls $package_key $package_url
	}
    }

    ad_proc -public get_sw_url {
	-package_key:required
    } {
	Returns the cached url of a site-wide package
    } {
	if {[catch {set url [nsv_get ml_cached_urls $package_key]}]} {
	    # no url found
	    return
	}
	return $url
    }

    ad_proc -public text_contains_tcl_code {
	-text:required
    } {
	Checks if text contains malicious inline-tcl code (adp-style)
    } {
	if {[regexp {<%[^=](.*?)%>} $text match tcl_code] || [regexp {<%=.*?(\[.*?)%>} $text match tcl_code]} {
	    return 1
	}
	return 0
    }

    ad_proc -public check_valid_vars {
	-text:required
	-valid_vars:required
    } {
	Checks if the template vars used in a text are all valid
	in the sense that they all can be found the provided list.
    } {
	# get list of used variables
	set current_index 0
	while {[regexp -indices -start $current_index [template::adp_variable_regexp] $text match match found_variable]} {
	    set current_index [expr [lindex $found_variable 1] + 1]
	    set user_variable [string range $text [lindex $found_variable 0] [lindex $found_variable 1]]
	    set used_vars($user_variable) 1
	}

	set errors ""
	# check if used variables are all valid
	foreach used_var [array names used_vars] {
	    if {[lsearch -exact $valid_vars $used_var] == -1} {
		# used var not a valid one
		lappend errors $used_var
	    }
	}
	return $errors
    }

    ad_proc -public send_mail {
	-user_id:required
	-from_email:required
	-subject:required
	-body:required
	-mime_type:required
	-charset:required
	{-link ""}
	{-new_password ""}
    } {
	Sends the confirmation/welcome mail to a user
    } {
	if {[empty_string_p $body]} {
	    # don't send any mail if message is empty
	    return
	}

	db_1row get_user_data {}
	if {[string equal $user_first_names "first_names"]} {
	    set user_first_names ""
	}
	if {[string equal $user_last_name "last_name"]} {
	    set user_last_name ""
	    set user_name $user_first_names
	} else {
	    set user_name "$user_first_names $user_last_name"
	}

	# generate the x-variable that's needed to verify the users identity
	# when he clicks the link
	set x [ns_sha1 "$user_id $user_password $user_salt"]
	append link "&[export_url_vars x]"

	# if user is already known to the system and not newly generated,
	# leave password blank so that a <if @user_password@ not nil> in
	# the mail text will not get executed (intended to provide
	# the user_password in the email to new users)
	set user_password $new_password

	# we have to use noquote-substitution for plaintext emails
	if {$mime_type == "text/plain"} {
	    while {[regsub -all [template::adp_variable_regexp] $body {\1@\2;noquote@} body]} {}
	}

	# substitute template variables in confirmation email
	variable ::template::parse_level
	lappend ::template::parse_level [info level]
	set error_p 0
	with_catch __errmsg {
	    set __result [template::adp_compile -string $body]
	    template::adp_eval __result
	    set body $__adp_output
	} {
	    # substitution failed
	    set error_p 1
	}
	template::util::lpop ::template::parse_level

	set to_addr(email) [list $user_email]
	set to_addr(user_id) [list $user_id]
	set to_addr(name) [list $user_name]

	if {!$error_p} {
	    # now send the mail
	    ns_log notice "mime_type: $mime_type / charset: $charset"
	    mail_job::send_mail -to_addr [array get to_addr] -from_addr $from_email -subject $subject -body $body -mime_type $mime_type -charset $charset -package_id [ad_conn package_id]
	}
	return $error_p
    }

    ad_proc -public valid_x_field_p {
	-user_id:required
	-x_field:required
    } {
	Verifies the x-field in the users email
    } {
	db_1row get_user_data {}

	# generate the expected x-variable
	set expected_x [ns_sha1 "$user_id $user_password $user_salt"]

	# Check if both values are the same and return t or f
	return [string equal $x_field $expected_x]
    }

    ad_proc -public check_valid_emails {
	-emails:required
    } {
	Checks the csv-list of emails for valid email addresses
	and returns list of invalid emails
    } {
	# generate tmp-file for csv-processing
	set tmp_file [ns_tmpnam]
	set fd [open $tmp_file w]
	puts $fd $emails
	close $fd
	set fd [open $tmp_file r]

	set invalid_list ""

	# process every line of the email list
	while {[ns_getcsv $fd line] > 0} {
	    # grap the email of the current line
	    util_unlist $line email user_first_names user_last_name

	    if {![util_email_valid_p $email]} {
		# invalid email address
		lappend invalid_list $email
	    }
	}
	return $invalid_list
    }
}
